void motor_control_initialise()
{  
  // initialise motor driver
  m1forward = true;
  m2forward = true;
  md = DualVNH5019MotorShield(M1INA,M1INB,M1EN,M1CS,M2INA,M2INB,M2EN,M2CS);
  md.init();
  md.setM1Brake(400);
  md.setM2Brake(400);
  
  // initialise switches
  pinMode(switchHorizontal_PIN,INPUT); // enable input on switchHorizontal's pin
  pinMode(switchVertical_PIN,INPUT); // enable input on switchVertical's pin
  //digitalWrite(switchHorizontal_PIN,HIGH); // enable internal pull-up resistor on switchHorizontal's pin
  //digitalWrite(switchVertical_PIN,HIGH); // enable internal pull-up resistor on switchVertical's pin 
  attachInterrupt(switchHorizontal_INT, switchHorizontal_closed, CHANGE); // interrupt on switchHorizontal closed
  attachInterrupt(switchVertical_INT, switchVertical_closed, CHANGE); // interrupt on switchVertical closed
  switchHorizontal = digitalRead(switchHorizontal_PIN); // original state of switchHorizontal
  switchVertical = digitalRead(switchVertical_PIN); // original state of switchVertical
  
  // initialise hall sensors
  attachInterrupt(rotateHorizontal_INT, rotateHorizontal_inc, RISING); // horizontal motor hall sensor
  attachInterrupt(rotateVertical_INT, rotateVertical_inc, RISING); // vertical motor hall sensor
  rotateHorizontal = 0;
  rotateVertical = 0;
    
  Serial.println("Motor Control Initialised");
  
  // if both switches are not closed => not in home position, call recalibrate 
  //recalibrate();
  
  print_limit();
  print_rotate();
  delay(3000);
  
  // moveM2(100) - moves M2 UP by 100 pulses
  // moveM1(100) - moves M1 ANTICLOCKWISE (away from switch) by 100 pulses
  
  //moveToAngle(360.0f,110.0f); // full range of motion
  //moveToAngle(0.0f,0.0f); // home
  
  recalibrate();
}

void recalibrate()
{
  // TO DO - Save this offset for debugging
  
  // bring vertical down
  while(!switchVertical)
  {
    moveM2(-1);
    delay(250);
  }
  Serial.print("switchVertical initial offset: ");
  Serial.println(rotateVertical);
  rotateVertical = 0;
  // move vertical up so it will not crash into the machine while recalibrating horizontal
  if(!switchHorizontal) {
    moveM2(wormFactor);
    delay(250);
  }
  
  // recalibrate horizontal
  while(!switchHorizontal)
  {
   moveM1(-1);
   delay(250);
  }  
  Serial.print("switchHorizontal offset: ");
  Serial.println(rotateHorizontal);
  rotateHorizontal = 0;
  Serial.println("switchHorizontal Recalibrated");
  
  // recalibrate vertical
  while(!switchVertical)
  {
    moveM2(-1);
    delay(250);
  }
  Serial.print("switchVertical offset: ");
  Serial.println(rotateVertical);
  rotateVertical = 0;
  Serial.println("switchVertical Recalibrated");
}

void print_limit()
{
  Serial.print("switchHorizontal: ");
  Serial.println(switchHorizontal);
  Serial.print("switchVertical: ");
  Serial.println(switchVertical);
}

void print_rotate()
{
  delay(200);
  Serial.print("rotateHorizontal: ");
  Serial.println(rotateHorizontal,DEC);
  Serial.print("rotateVertical: ");
  Serial.println(rotateVertical,DEC);
  delay(200);
}

void moveToAngle(float horizontal, float vertical) {  
  // check valid range
  if (horizontal < 0.f || horizontal > 360.f) return;
  if (vertical < 0.f || vertical > 110.f) return;
  
  // horizontal calculation
  int desiredHorizontal =  constrain(int(horizontal*float(maxHorizontal)/360.f),0,maxHorizontal); // desired horizontal pulses from switch
  int targetHorizontal = constrain(desiredHorizontal-rotateHorizontal,-maxHorizontal,maxHorizontal); // target horizontal pulses to move
  
  // vertical calculation (including wormCorrection factor)
  int desiredVertical = constrain(int(vertical*float(maxVertical)/110.f),0,maxVertical); // desired vertical pulses from switch
  // Note: will underestimate targetWormCorrection (floor) which means targetVertical might be slightly too large (beyond maximum vertical limit by 1 pulse)
  int targetWormCorrection = int((long(desiredHorizontal)*long(wormFactor))/long(maxHorizontal));
  int targetVertical = desiredVertical-targetWormCorrection-rotateVertical;
  
  // debugging
  /*
  Serial.print("desiredHorizontal= ");
  Serial.println(desiredHorizontal);
  Serial.print("wormFactor= ");
  Serial.println(wormFactor);
  Serial.print("desiredVertical= ");
  Serial.println(desiredVertical);
  Serial.print("targetWormCorrection= ");
  Serial.println(targetWormCorrection);
  Serial.print("rotateVertical= ");
  Serial.println(rotateVertical);
  Serial.print("targetVertical= ");
  Serial.println(targetVertical);
  return;
  */
    
  // actual limits at target horizontal are -targetWormCorrection < rotateVertical < maxVertical-targetWormCorrection
  // actual limits at current horizontal are -currentWormCorrection < rotateVertical < maxVertical-currentWormCorrection
  int currentWormCorrection = int((long(rotateHorizontal)*long(wormFactor))/long(maxHorizontal));
  
  // order movements to keep vertical between limit switch and max vertical limit
  if (targetHorizontal > 0 && targetVertical > 0) {
    moveM1(targetHorizontal);
    moveM2(targetVertical);
  }
  else if (targetHorizontal < 0 && targetVertical < 0) {
    moveM1(targetHorizontal);
    moveM2(targetVertical);
  }
  else if (targetHorizontal < 0 && targetVertical > 0) { // difficult case
    if (rotateVertical + targetVertical < maxVertical - currentWormCorrection) { // ok to move vertically now
      moveM2(targetVertical);
      moveM1(targetHorizontal);
    } else if (rotateVertical > -targetWormCorrection) { // will not hit limit switch before desination
      moveM1(targetHorizontal);
      moveM2(targetVertical);
    } else { // cannot manoeuvre in 2 moves
      targetVertical -= maxVertical - rotateVertical - currentWormCorrection; // targetVertical > 0, rotateVertical < -targetWormCorrection < 0, currentWormCorrection > 0
      moveM2(maxVertical -rotateVertical - currentWormCorrection); // move to current max vertical limit
      moveM1(targetHorizontal);
      moveM2(targetVertical); // move remainder of vertical motion
    }
  }
  else { // targetHorizontal > 0 && targetVertical < 0 // difficult case
    if (rotateVertical + targetVertical > -currentWormCorrection) { // ok to move vertically now
      moveM2(targetVertical);
      moveM1(targetHorizontal);
    } else if (rotateVertical < maxVertical - targetWormCorrection) { // will not go beyond max vertical limit before destination
      moveM1(targetHorizontal);
      moveM2(targetVertical);
    } else { // cannot manoeuvre in 2 moves
      targetVertical -= -rotateVertical - currentWormCorrection; // targetVertical < 0, rotateVertical > maxVertical-targetWormCorrection > maxVertical-wormFactor > 0, currentWormCorrection > 0
      moveM2(-rotateVertical - currentWormCorrection); // move to switch
      moveM1(targetHorizontal);
      moveM2(targetVertical); // move remainder of vertical motion
    }
  }
}

void moveM1(int pulses) {
  if (pulses== 0) return;
  int target = rotateHorizontal + pulses;
  int gain = 8;
  int thresholdForward = 80;
  int thresholdBackward = 80;
  if (pulses > 0) {
    // ramp forward    
    m1forward = true;
    while (target > rotateHorizontal) {
      // target - pulses = initial rotateHorizontal
      // rotateHorizontal - (target-pulses) = pulses moved 
      // target - rotateHorizontal = pulses remaining
      md.setM1Speed(constrain(max(thresholdForward,gain*min(rotateHorizontal-(target-pulses),target-rotateHorizontal)),0,400));
      // check maximum range of M1 not reached and maximum range of M2 not reached (Note: must add wormFactor correction)
      if ((rotateHorizontal >= maxHorizontal) || (int(long(rotateVertical)+((long(rotateHorizontal)*long(wormFactor))/long(maxHorizontal))) >= maxVertical)){
        md.setM1Speed(0);
        md.setM1Brake(400);
        break;
      }      
      delay(1);
    }
    md.setM1Speed(0);
    md.setM1Brake(400);
    delay(100); // let motor spin down
    switchHorizontal = digitalRead(switchHorizontal_PIN); // check if switchHorizontal is still pressed
    switchVertical = digitalRead(switchVertical_PIN); // check if switchVertical is still pressed (can be unpressed by M1 moving positively)
  } else {
    // ramp backward
    switchHorizontal = digitalRead(switchHorizontal_PIN); // check if switchHorizontal is still pressed    
    m1forward = false;
    while (target < rotateHorizontal) {
      // target - pulses = initial rotate 1
      // (target-pulses) - rotateHorizontal = pulses moved (positive)
      // rotateHorizontal - target = pulses remaining (positive)
      md.setM1Speed(constrain(min(-1*thresholdBackward,-1*gain*min(target-pulses-rotateHorizontal,rotateHorizontal-target)),-400,0));      
      // check limits of M1 not reached
      if (digitalRead(switchHorizontal_PIN) == HIGH || switchHorizontal)
      {
        md.setM1Speed(0);
        md.setM1Brake(400);
        switchHorizontal = true;
        break;
      }
      // check limits of M2 not reached
      if (digitalRead(switchVertical_PIN) == HIGH || switchVertical)
      {
        md.setM1Speed(0);
        md.setM1Brake(400);
        switchVertical = true;
        break;
      }
      delay(1);
    }
    md.setM1Speed(0);
    md.setM1Brake(400);
    delay(100); // let motor spin down
  }
}

void moveM2(int pulses) {
  if (pulses== 0) return;
  int target = rotateVertical + pulses;
  int gain = 7; // lower gain to prevent M2 approach limit switch too quickly
  int thresholdForward = 110; // M2 faces higher resistance than M2
  int thresholdBackward = 90;
  if (pulses > 0) {
    // ramp forward
    m2forward = true;
    while (target > rotateVertical) {
      // target - pulses = initial rotateVertical
      // rotateVertical - (target-pulses) = pulses moved 
      // target - rotateVertical = pulses remaining
      md.setM2Speed(constrain(max(thresholdForward,gain*min(rotateVertical-(target-pulses),target-rotateVertical)),0,400));
      // check maximum range not reached (Note: must add wormFactor correction)
      if (int(long(rotateVertical)+((long(rotateHorizontal)*long(wormFactor))/long(maxHorizontal))) >= maxVertical){
        md.setM2Speed(0);
        md.setM2Brake(400);
        break;
      }        
      delay(1);
    }
    md.setM2Speed(0);
    md.setM2Brake(400);
    delay(100); // let motor spin down
    switchVertical = digitalRead(switchVertical_PIN); // check if switchVertical is still pressed 
  } else {
    // ramp backward
    switchVertical = digitalRead(switchVertical_PIN); // check if switchVertical is still pressed (can be unpressed by M1 moving positively due to worm drive)
    m2forward = false;
    while (target < rotateVertical) {
      // target - pulses = initial rotateVertical
      // (target-pulses) - rotateVertical = pulses moved (positive)
      // rotateVertical - target = pulses remaining (positive)
      md.setM2Speed(constrain(min(-1*thresholdBackward,-1*gain*min(target-pulses-rotateVertical,rotateVertical-target)),-400,0));      
      // check limits not reached
      if (digitalRead(switchVertical_PIN) == HIGH || switchVertical)
      {
        md.setM2Speed(0);
        md.setM2Brake(400);
        switchVertical = true;
        break;
      }
      delay(1);
    }
    md.setM2Speed(0);
    md.setM2Brake(400);
    delay(100); // let motor spin down
  }
}

// Interrupt Functions
void switchHorizontal_closed()
{
  switchHorizontal = true;
}

void switchVertical_closed()
{
  switchVertical = true;
}

void rotateHorizontal_inc()
{
  if(m1forward) {
    rotateHorizontal++;
  } else {
    rotateHorizontal--;
  }
}
 
void rotateVertical_inc()
{
  if(m2forward) {
    rotateVertical++;
  } else {
    rotateVertical--;
  }
}
