#include <MemoryFree.h>
#include <BigNumber.h>
#include <BigNumberMath.h>

/* ****************************** Motor Control Variables ******************************* */
// pololu motor driver shield
#include <DualVNH5019MotorShield.h>
#define M1INA 4 // remapped
#define M1INB 5 // remapped
#define M1EN 6
#define M1CS A0
#define M2INA 7
#define M2INB 8
#define M2EN 12
#define M2CS A1
//#define M1PWM "9" // can not be remapped
//#define M2PWM "10" // can not be remapped
DualVNH5019MotorShield md; // motor driver object

// home limit switches
#define switchHorizontal_PIN 18 // horizontal switch pin
#define switchVertical_PIN 19   // vertical switch pin
#define switchHorizontal_INT 5  // pin 18 interrupt
#define switchVertical_INT 4    // pin 19 interrupt
volatile boolean switchHorizontal; // state of horizontal switch (1 = closed, 0 = open)
volatile boolean switchVertical;   // state of vertical switch (1 = closed, 0 = open)

// maximum limits
#define maxHorizontal 840 // pulses for 360 degree horizontal movement
#define maxVertical 680 // pulses for 110 degree vertical movement

// worm drive correction
#define wormFactor 147 // pulses moved vertically per 360 degrees horizontally

// hall sensor count
// TO DO - consider using internal pull-up resistors as in the home limit switch case above
#define rotateHorizontal_INT 0
#define rotateVertical_INT 1
volatile int rotateHorizontal;
volatile int rotateVertical;

// motor direction (1 = forward, 0 = backward)
volatile boolean m1forward;
volatile boolean m2forward;
/* ************************************************************************************** */

/* ****************************** Solar Tracking Variables ****************************** */
float SunsAltitude, SunsAzimuth;

// location
float latitude = 39.19;
float longitude = -78.16;
int timezone = 0;

// time
int month = 9;
int year = 2012;
int day = 11;
int hour = 11;
int minute = 00;
int second = 00;

// BigNumber library settings
int calculationScale=30;
int calculationSpeed=0;// 0 for slow, 1 for fast (but less accurate)
int sinePrec=20;
int arcSinePrec=10;
int cosinePrec=20;
int arcTanPrec=10;

//If you live in the southern hemisphere, it would probably be easier
//to make north as the direction where the azimuth equals
//0 degrees. To do so, switch the 0 below to 180.  
float northOrSouth = 0;
/* ************************************************************************************** */

void setup() {
  // Prototyping Delay
  delay(15000);
  
  Serial.begin(9600);
  
  motor_control_initialise();
  
  //moveToAngle(360.f,110.f);
  //delay(5000);
  //moveToAngle(0.f,0.f);
  //recalibrate();
  
  for (int i = 11; i < 24; i++) {
  hour=i;
  print_input();  
  findSunsAltAndAzOne(year, month, day, timezone, hour, minute, second, latitude, longitude);
  print_output();
  
  float verticalAngle = 90.f-SunsAltitude; // angle from vertically above (zenith)
  float horizontalAngle = 180.f-SunsAzimuth; // angle from true north
  
  Serial.print("verticalAngle="); Serial.println(verticalAngle);
  Serial.print("horizontalAngle="); Serial.println(horizontalAngle);
  
  moveToAngle(horizontalAngle,verticalAngle);
  //delay(1000);
  
  Serial.print("freeMemory()="); Serial.println(freeMemory());
  }
  
  moveToAngle(0.f,0.f);
  
  recalibrate();
  
}

void loop() {
  // Select pseudorandom input
  //year = int(random(2000,2051));
  //month = int(random(13));
  //day = int(random(0,25));
  //timezone = int(random(-5,6));
  //hour = int(random(25));
  //minute = int(random(61));
  //second = int(random(61));
  //latitude = float(random(360001)/1000.f);
  //longitude = float(random(180001)/1000.f);
  
}

void print_input() {  
  Serial.print("Year: "); Serial.println(year);
  Serial.print("Month:" ); Serial.println(month);
  Serial.print("Day: "); Serial.println(day);
  Serial.print("Timezone: "); Serial.println(timezone);
  Serial.print("Hour: "); Serial.println(hour);
  Serial.print("Minute: "); Serial.println(minute);
  Serial.print("Second: "); Serial.println(second);
  Serial.print("Latitude: "); Serial.println(latitude);
  Serial.print("Longitude: "); Serial.println(longitude);
}

void print_output() {
  Serial.print("SunsAltitude: "); Serial.println(SunsAltitude);
  Serial.print("SunsAzimuth: "); Serial.println(SunsAzimuth); 
}
